import * as path from 'path';
import * as webpack from 'webpack';
import { createHash } from 'crypto';
import jsStringEscape from 'js-string-escape';
import { initializeBuildOptions, getSerializedStore } from './store.esm.js';
import '@component-controls/core';
import '@component-controls/config';

class LoaderPlugin {
    constructor(options) {
        const hash = createHash('md5')
            .update(new Date().getTime().toString())
            .digest('hex');
        this.compilationHash = `__${hash.substring(0, 5)}__`;
        this.options = Object.assign({}, options);
    }
    apply(compiler) {
        initializeBuildOptions(compiler.context, this.options.config, this.options.defaultConfigPath, this.options.buildConfig);
        this.replaceRuntimeModule(compiler);
        compiler.hooks.compilation.tap(LoaderPlugin.pluginName, compilation => {
            var _a, _b;
            if (typeof ((_a = compilation.hooks.processAssets) === null || _a === void 0 ? void 0 : _a.tapPromise) === 'function') {
                compilation.hooks.processAssets.tapPromise({
                    name: LoaderPlugin.pluginName,
                    stage: webpack.Compilation.PROCESS_ASSETS_STAGE_ADDITIONS,
                }, async (assets) => {
                    const jsFiles = [];
                    Object.keys(assets)
                        .filter(fileName => {
                        return fileName.endsWith('.js');
                    })
                        .forEach(fileName => jsFiles.push(fileName));
                    for (let i = 0; i < jsFiles.length; i += 1) {
                        await this.replaceSource(compilation, jsFiles[i]);
                    }
                });
            }
            else if (typeof ((_b = compilation.hooks.optimizeChunkAssets) === null || _b === void 0 ? void 0 : _b.tapPromise) === 'function') {
                compilation.hooks.optimizeChunkAssets.tapPromise(LoaderPlugin.pluginName, async (chunks) => {
                    const jsFiles = [];
                    chunks.forEach(chunk => {
                        chunk.files
                            .filter((fileName) => fileName.endsWith('.js'))
                            .forEach((fileName) => jsFiles.push(fileName));
                    });
                    for (let i = 0; i < jsFiles.length; i += 1) {
                        await this.replaceSource(compilation, jsFiles[i]);
                    }
                });
            }
        });
    }
    replaceRuntimeModule(compiler) {
        const nmrp = new webpack.NormalModuleReplacementPlugin(/story-store-data\.js$/, (resource) => {
            var _a;
            //webpack 4 or webpack 5
            const plugins = resource.loaders || ((_a = resource.createData) === null || _a === void 0 ? void 0 : _a.loaders);
            if (plugins) {
                plugins.push({
                    loader: path.join(__dirname, 'runtimeLoader.js'),
                    options: JSON.stringify({
                        compilationHash: this.compilationHash,
                    }),
                });
            }
        });
        nmrp.apply(compiler);
    }
    async replaceSource(compilation, file) {
        const placeholder = `__COMPILATION_HASH__${this.compilationHash}`;
        const source = compilation.assets[file];
        const sources = webpack.sources || require('webpack-sources');
        const placeholderPos = source.source().indexOf(placeholder);
        if (placeholderPos > -1) {
            const store = await getSerializedStore();
            const newContent = this.options.escapeOutput
                ? jsStringEscape(store)
                : store;
            const newSource = new sources.ReplaceSource(source, file);
            newSource.replace(placeholderPos, placeholderPos + placeholder.length - 1, newContent, file);
            compilation.assets[file] = newSource;
        }
    }
}
LoaderPlugin.pluginName = 'component-controls-loader-plugin';
module.exports = LoaderPlugin;

export { LoaderPlugin };
